﻿import os
import appModuleHandler
import keyboardHandler
import ui
import api
import textInfos
import re
import json

dir = os.path.dirname(__file__)
with open(os.path.join(dir, "margin"), 'r') as f:
  margin = json.load(f)
origin = margin

def reform(brl):
  global margin, origin
  if origin != margin:
    with open(os.path.join(dir, "margin"), 'w') as f:
      json.dump(margin, f)
  consonant = "opmqdfncklrjeabigh"
  vowel = "><!5w0%(vuxz:*/|)+{st?.y937$}=_&8~46"
  tone = "'1`\"a"
  cut = " 2"

  brl = brl.replace("-\r", "- \r")
  brl = brl.replace("\n ", "\v ")
  brl = brl.strip("\r\n")
  brl = brl.replace("\v", "\n")
  paragraph = brl.split("\r\n")
  brl = ""

  for p in paragraph:
    pos = margin
    while len(p) > margin:
      ch = p[pos]
      if ch == " ":
        brl += p[:pos] + "\r\n"
        p = p[pos+1:]
        pos = margin
        continue
      if p[pos-1] in cut:
        brl += p[:pos] + "\r\n"
        p = p[pos:]
        pos = margin
        continue
      if p[pos-1] in tone and (ch in consonant and p[pos+1] in vowel) or (p[pos-1] == 'a' and p[pos-2] in vowel and ch in vowel):
        brl += p[:pos] + "\r\n"
        p = p[pos:]
        pos = margin
        continue
      pos -= 1
      if pos < 3:
        brl += p[:margin] + "\r\n"
        p = p[margin:]
        pos = margin
    brl += p + "\r\n"
  return brl.rstrip("\r\n")

def getobj():
  obj = api.getFocusObject()
  treeInterceptor = obj.treeInterceptor
  if hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough:
    obj = treeInterceptor
  return obj

class AppModule(appModuleHandler.AppModule):
  def script_format(self, gesture):
    focus = getobj()
    try:
      info = focus.makeTextInfo(textInfos.POSITION_SELECTION)
    except (RuntimeError, NotImplementedError):
      info = None
    if not info or info.isCollapsed:
      ui.message("沒有選取範圍")
    else:
      brl = reform(info.text.replace('\r\n', ''))
      api.copyToClip(brl)
      keyboardHandler.KeyboardInputGesture.fromName("delete").send()
      keyboardHandler.KeyboardInputGesture.fromName("control+v").send()
      ui.message(_("重排完成"))

  def script_incrmargin(self, gesture):
    global margin
    if margin >= 42:
      ui.message("寬度已達上限 42")
    else:
      margin += 1
      ui.message("寬度 " + str(margin))

  def script_descrmargin(self, gesture):
    global margin
    if margin <= 14:
      ui.message("寬度已達下限 14")
    else:
      margin -= 1
      ui.message("寬度 " + str(margin))

  def script_help(self, gesture):
    ui.browseableMessage("""
      alt+向左游標: 減少寬度
      alt+向右游標: 增加寬度
      alt+c: 點字轉國字
      control+r: 重排選取內容
      NVDA+space: 往下尋找空白行
      shift+NVDA+space: 往上尋找空白行
      """, "CCBE 按鍵說明")

  def script_b2c(self, gesture):
    ch = []
    with open('char.dic', newline='') as file:
      for i in file:
        ch.append(i.split('\t'))
    ph = []
    with open('phrase.dic', newline='') as file:
      for i in file:
        ph.append(i.split('\t'))

  def script_blank(self, gesture):
    obj = getobj()
    info = obj.makeTextInfo(textInfos.POSITION_CARET)
    info.collapse()
    while info.move(textInfos.UNIT_LINE, 1):
      info.expand(textInfos.UNIT_LINE)
      if info.text == '\n':
        info.updateCaret()
        return
    ui.message("沒有空白行")

  def script_backblank(self, gesture):
    obj = getobj()
    info = obj.makeTextInfo(textInfos.POSITION_CARET)
    info.collapse()
    while info.move(textInfos.UNIT_LINE, -1):
      info.expand(textInfos.UNIT_LINE)
      if info.text == '\n':
        info.updateCaret()
        return
    ui.message("沒有空白行")

  script_format.__doc__ = "點字排版"
  script_incrmargin.__doc__ = "增加寬度"
  script_descrmargin.__doc__ = "減少寬度"
  script_b2c.__doc__ = "點字轉國字"
  script_blank.__doc__ = "往下尋找空白行"
  script_backblank.__doc__ = "往上尋找空白行"
  script_help.__doc__ = "CCBE 按鍵說明"

  __gestures = { "kb:control+r":"format",
                 "kb:alt+rightArrow":"incrmargin",
                 "kb:alt+leftArrow":"descrmargin",
                 "kb:alt+c":"b2c",
                 "kb:alt+f1":"help",
                 "kb:NVDA+space":"blank",
                 "kb:shift+NVDA+space":"backblank"}